@@ -64,8 +64,8 @@ namespace ojph {
     // index is (c_q << 8) + (rho << 4) + eps
     // data is  (cwd << 8) + (cwd_len << 4) + eps
     // table 0 is for the initial line of quads
-    static ui32 vlc_tbl0[2048] = { 0 };
-    static ui32 vlc_tbl1[2048] = { 0 };
+    static ui32 vlc_tbl0[2048];
+    static ui32 vlc_tbl1[2048];
 
     //UVLC encoding
     static ui32 ulvc_cwd_pre[33];
@@ -218,18 +218,18 @@ namespace ojph {
     }
 
     /////////////////////////////////////////////////////////////////////////
-    bool initialize_tables_avx2() {
-      if (get_cpu_ext_level() >= X86_CPU_EXT_LEVEL_AVX2) {
-        bool result;
-        result = vlc_init_tables();
-        result = result && uvlc_init_tables();
-        return result;
-      }
-      return false;
-    }
+    static bool tables_initialized = false;
 
     /////////////////////////////////////////////////////////////////////////
-    static bool tables_initialized = initialize_tables_avx2();
+    bool initialize_block_encoder_tables_avx2() {
+      if (!tables_initialized) {
+        memset(vlc_tbl0, 0, 2048 * sizeof(ui32));
+        memset(vlc_tbl1, 0, 2048 * sizeof(ui32));
+        tables_initialized = vlc_init_tables();
+        tables_initialized = tables_initialized && uvlc_init_tables();
+      }
+      return tables_initialized;
+    }
 
     /////////////////////////////////////////////////////////////////////////
     //
